#!/usr/bin/env python3

from typing import TYPE_CHECKING, Callable, NoReturn
import sys
from .prompts import (
    REPLY_OK_IF_YOU_READ_TEMPLATE,
    REPLY_OK_IF_YOU_READ_TEMPLATE_SPLITTED_FIRST,
    REPLY_OK_IF_YOU_READ_TEMPLATE_SPLITTED_CONTINUED,
)


if TYPE_CHECKING:
    from datetime import datetime
    from langchain.docstore.document import Document


def convert_str_slice_notation_to_slice(str_slice: str) -> slice:
    # '1:3' -> slice(1, 3)
    # '1:' -> slice(1, None)
    # ':3' -> slice(None, 3)
    # ':' -> slice(None, None)
    # '3' -> slice(3)
    # '1:8:2' -> slice(1, 8, 2)
    # start
    def int_or_none(s: str) -> int | None:
        try:
            return int(s)
        except ValueError:
            return None

    return slice(*list(map(int_or_none, str_slice.split(':'))))
    # if str_slice.startswith(':'):
    #     start = None
    # else:
    #     start = int(str_slice.split(':')[0])
    # # stop
    # if str_slice.endswith(':'):
    #     stop = None
    # else:
    #     try:
    #         stop = int(str_slice.split(':')[1])
    #     except IndexError:
    #         stop = None
    # # step
    # if len(str_slice.split(':')) == 3:
    #     step = int(str_slice.split(':')[-1])
    # else:
    #     step = None
    # return slice(start, stop, step)


def get_token_count(s: str, model_name: str = 'gpt-3.5-turbo') -> int:
    from tiktoken import encoding_for_model

    enc = encoding_for_model(model_name)
    tokenized_text = enc.encode(s)

    # calculate the number of tokens in the encoded text
    return len(tokenized_text)


def get_word_count(s: str) -> int:
    return len(s.split())


def format_date(dt: 'datetime') -> str:
    return dt.strftime('%Y-%m-%d')


def pymupdf_doc_page_info(document: 'Document') -> str:
    metadata = document.metadata
    if 'page_number' in metadata:
        return f', Page {metadata["page_number"]}/{metadata["total_pages"]}'
    else:
        return ''


def html_source_info(document: 'Document') -> str:
    metadata = document.metadata
    if 'source' in metadata:
        return f', Title: {metadata["title"]}, Source: {metadata["source"]}'
    else:
        return ''

def general_document_source_info(document: 'Document') -> str:
    metadata = document.metadata
    if 'source' in metadata:
        return f', Source: {metadata["source"]}'
    else:
        return ''


def save_str_to_tempfile(s: str, suffix: str = '.txt') -> str:
    import tempfile

    with tempfile.NamedTemporaryFile(mode='w', suffix=suffix, delete=False) as f:
        f.write(s)
        return f.name


def open_file(path: str):
    # if on macOS, open with default app
    if sys.platform == 'darwin':
        import subprocess

        subprocess.call(('open', path))
    # if on Linux, open with default app
    elif sys.platform.startswith('linux'):
        import subprocess

        subprocess.call(('xdg-open', path))
    # if on Windows, open with default app
    elif sys.platform == 'win32':
        import os

        os.startfile(path)
    else:
        raise NotImplementedError(f'Unsupported platform: {sys.platform}')


def deliver_prompts(
    what: str,
    documents: list['Document'],
    should_be_only_one_doc: bool = False,
    needs_splitting: bool = False,
    copy: bool = True,
    edit: bool = False,
    chunk_size: int = 2000,
    extra_chunk_info_fn: Callable[['Document'], str] = lambda doc: '',
    dry_run: bool = False,
    parts: list[int] | None = None,
):
    from langchain.prompts import PromptTemplate

    def deliver_single_doc(document: 'Document'):
        prompt = PromptTemplate.from_template(REPLY_OK_IF_YOU_READ_TEMPLATE)
        content = document.page_content
        formatted_prompt = prompt.format(what=what, content=content)

        def edit_prompt(formatted_prompt: str = formatted_prompt):
            formatted_prompt_path = save_str_to_tempfile(
                formatted_prompt, suffix='.txt'
            )
            open_file(formatted_prompt_path)
            print(
                f'Please edit the prompt at {formatted_prompt_path} and copy it yourself.',
                file=sys.stderr,
            )
            return

        if edit and not dry_run:
            edit_prompt()
            return
        if copy:
            print(
                f'Word Count: {get_word_count(formatted_prompt)}, Char count: {len(formatted_prompt)}{extra_chunk_info_fn(document)}',
                file=sys.stderr,
            )
            if not dry_run:
                import pyperclip

                pyperclip.copy(formatted_prompt)
                print('Prompt copied to clipboard.', file=sys.stderr)
        else:
            print(formatted_prompt)

    def deliver_multiple_docs(documents: list['Document']):
        if edit:
            print(f'Please copy the prompts after each edits.', file=sys.stderr)
        for i, doc in enumerate(documents):
            num_chunks = len(documents)
            if i == 0:
                prompt = PromptTemplate.from_template(
                    REPLY_OK_IF_YOU_READ_TEMPLATE_SPLITTED_FIRST
                )
            else:
                prompt = PromptTemplate.from_template(
                    REPLY_OK_IF_YOU_READ_TEMPLATE_SPLITTED_CONTINUED
                ).partial(x=str(i + 1))
            content = doc.page_content
            formatted_prompt = prompt.format(what=what, content=content)
            if dry_run:
                print(
                    f'Press Enter to copy prompt {i+1}/{num_chunks}. Word Count: {get_word_count(formatted_prompt)}, Char count: {len(formatted_prompt)}{extra_chunk_info_fn(doc)}: '
                )
                continue
            if edit:
                input(
                    f'Press Enter to edit prompt {i+1}/{num_chunks}. Word Count: {get_word_count(formatted_prompt)}, Char count: {len(formatted_prompt)}{extra_chunk_info_fn(doc)}: '
                )
                formatted_prompt_path = save_str_to_tempfile(
                    formatted_prompt, suffix='.txt'
                )
                open_file(formatted_prompt_path)
                continue
            input(
                f'Press Enter to copy prompt {i+1}/{num_chunks}. Word Count: {get_word_count(formatted_prompt)}, Char count: {len(formatted_prompt)}{extra_chunk_info_fn(doc)}: '
            )
            import pyperclip

            pyperclip.copy(formatted_prompt)

    if dry_run:
        print(
            f'Dry running. Nothing will be copied to your clipboard, and you don\''
            't need to press Enter to move forward.'
        )
    if needs_splitting:
        from langchain.text_splitter import TokenTextSplitter

        splitter = TokenTextSplitter(encoding_name='cl100k_base', chunk_size=chunk_size)
        splitted = splitter.split_documents(documents)
        if parts:
            len_splitted = len(splitted)
            parts = list({part for part in parts if 0 <= part - 1 < len_splitted})
            print(
                f'Selecting {len(parts)} parts out of {len_splitted}.', file=sys.stderr
            )
            print(f'Using parts: {parts}', file=sys.stderr)
            splitted = [splitted[i - 1] for i in parts]
        deliver_multiple_docs(splitted)

    elif should_be_only_one_doc:
        deliver_single_doc(documents[0])
    else:
        deliver_multiple_docs(documents)


def assert_never(a: NoReturn) -> NoReturn:
    raise RuntimeError("Should not get here")


def save_stdin_to_tempfile() -> str:
    # create a temp file and save stdin to it, and return the tempfile path
    import tempfile
    import shutil
    import sys

    with tempfile.NamedTemporaryFile(delete=False) as temp_file:
        with open(temp_file.name, 'w') as f:
            shutil.copyfileobj(sys.stdin, f)
        temp_file_path = temp_file.name
    return temp_file_path

def save_clipboard_to_tempfile() -> str:
    # create a temp file and save stdin to it, and return the tempfile path
    import tempfile
    import pyperclip
    import sys

    with tempfile.NamedTemporaryFile(delete=False) as temp_file:
        with open(temp_file.name, 'w') as f:
            f.write(pyperclip.paste())
        temp_file_path = temp_file.name
    return temp_file_path


#!/usr/bin/env python3
"""
Author : Xinyuan Chen <45612704+tddschn@users.noreply.github.com>
Date   : 2023-04-09
"""

import sys

from . import __version__
from .utils import (
    deliver_prompts,
    get_word_count,
    deliver_prompts,
    html_source_info,
    save_stdin_to_tempfile,
    save_clipboard_to_tempfile,
)
from .loaders import load_html
from .config import DEFAULT_HTML_WHAT
from .utils_argparse import get_get_prompt_base_arg_parser


def get_args():
    """Get command-line arguments"""

    parser = get_get_prompt_base_arg_parser(
        description='Get a prompt from html files'
    )

    parser.add_argument(
        'path', help='Paths to the html files, or stdin if not provided', metavar='PATH', type=str, default=None, nargs='*'
    )

    parser.add_argument(
        '-C', '--from-clipboard', help='Load text from clipboard', action='store_true'
    )

    parser.add_argument(
        '-w',
        '--what',
        help='Initial knowledge you want to insert before the PDF content in the prompt',
        type=str,
        default=DEFAULT_HTML_WHAT,
    )
    parser.add_argument(
        '-M',
        '--merge',
        help='Merge contents of all pages before processing',
        action='store_true',
    )

    args = parser.parse_args()
    if args.from_clipboard:
        args.path = [save_clipboard_to_tempfile()]
    elif not args.path:
        args.path = [save_stdin_to_tempfile()]
    return args


def main():
    """Make a jazz noise here"""

    args = get_args()

    print(f'Loading html file(s) from {args.path} ...', file=sys.stderr)
    docs = [load_html(p)[0] for p in args.path]
    texts = [doc.page_content for doc in docs]
    all_text = '\n'.join(texts)
    word_count = get_word_count((all_text))
    print(
        f'Loaded {len(docs)} pages. Word count: {word_count} Char count: {len(all_text)}',
        file=sys.stderr,
    )
    if args.merge:
        from langchain.docstore.document import Document

        merged = Document(
            page_content=all_text,
            # metadata=docs[0].metadata,
            # metadata={
            #     k: v for k, v in docs[0].metadata.items() if k not in {'page_number'}
            # },
        )
    documents = [merged] if args.merge else docs # type: ignore
    num_docs = len(documents)
    if args.split or word_count > args.chunk_size * 0.75:
        needs_splitting = True
    else:
        needs_splitting = False
    deliver_prompts(
        what=args.what,
        documents=documents, # type: ignore
        needs_splitting=needs_splitting,
        copy=args.copy,
        edit=args.edit,
        should_be_only_one_doc=True if num_docs == 1 else False,
        chunk_size=args.chunk_size,
        extra_chunk_info_fn=html_source_info,
        dry_run=args.dry_run,
        parts=args.parts,
    )


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

from typing import TYPE_CHECKING, Literal

if TYPE_CHECKING:
    from langchain.docstore.document import Document


def load_youtube_url(youtube_url: str) -> list['Document']:
    from langchain.document_loaders import YoutubeLoader

    loader = YoutubeLoader.from_youtube_channel(youtube_url, add_video_info=True)
    docs = loader.load()
    return docs


def load_pdf(pdf_path: str) -> list['Document']:
    from langchain.document_loaders import PyMuPDFLoader

    loader = PyMuPDFLoader(pdf_path)
    docs = loader.load()
    return docs


def load_url(urls: list[str], javascript: bool = False) -> list['Document']:
    from langchain.document_loaders import UnstructuredURLLoader, SeleniumURLLoader

    if javascript:
        loader_class = SeleniumURLLoader
        kwargs = {}
    else:
        loader_class = UnstructuredURLLoader
        # headers = {
        #     'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/94.0.4606.71 Safari/537.36'
        # }

        # kwargs = {'headers': headers}
        # You are using old version of unstructured. The headers parameter is ignored
        kwargs = {}

    from unstructured.partition.html import partition_html

    partition_html(url='https://mp.weixin.qq.com/s/FsrDnCFKGD-FzP5YD76tbA')
    loader = loader_class(urls=urls, **kwargs)
    docs = loader.load()

    return docs

def load_text(path: str, encoding: str | None = None) -> list['Document']:
    from langchain.document_loaders import TextLoader

    loader = TextLoader(path, encoding=encoding)
    docs = loader.load()
    return docs

def load_html(path: str, open_encoding: str | None = None, bs_kwargs: dict | None = None) -> list['Document']:
    from langchain.document_loaders import BSHTMLLoader

    loader = BSHTMLLoader(path, open_encoding=open_encoding, bs_kwargs=bs_kwargs)
    docs = loader.load()
    return docs

UnstructuredLoadingMode = Literal["single", "elements"] 

def load_word(path: str, mode: UnstructuredLoadingMode = "single") -> list['Document']:
    # UnstructuredWordDocumentLoader
    from langchain.document_loaders import UnstructuredWordDocumentLoader

    loader = UnstructuredWordDocumentLoader(path, mode=mode)
    docs = loader.load()
    return docs


#!/usr/bin/env python3
"""
Author : Xinyuan Chen <45612704+tddschn@users.noreply.github.com>
Date   : 2023-04-09
"""

import sys

from . import __version__
from .utils import (
    deliver_prompts,
    get_word_count,
    deliver_prompts,
    pymupdf_doc_page_info,
    convert_str_slice_notation_to_slice,
)
from .loaders import load_pdf
from .config import DEFAULT_PDF_WHAT
from .utils_argparse import get_get_prompt_base_arg_parser


def get_args():
    """Get command-line arguments"""

    parser = get_get_prompt_base_arg_parser(
        description='Get a prompt consisting the text content of a PDF file'
    )

    parser.add_argument(
        'pdf_path', help='Path to the PDF file', metavar='PDF Path', type=str
    )
    parser.add_argument(
        '-p',
        '--pages',
        help='Only include specified page numbers',
        type=int,
        nargs='+',
        default=None,
    )
    parser.add_argument(
        '-l',
        '--page-slice',
        help='Use Python slice syntax to select page numbers (e.g. 1:3, 1:10:2, etc.)',
        type=str,
        default=None,
    )
    parser.add_argument(
        '-M',
        '--merge',
        help='Merge contents of all pages before processing',
        action='store_true',
    )
    parser.add_argument(
        '-w',
        '--what',
        help='Initial knowledge you want to insert before the PDF content in the prompt',
        type=str,
        default=DEFAULT_PDF_WHAT,
    )

    args = parser.parse_args()
    return args


def main():
    """Make a jazz noise here"""

    args = get_args()

    print(f'Loading PDF from {args.pdf_path} ...', file=sys.stderr)
    docs = load_pdf(args.pdf_path)
    num_whole_pdf_pages = len(docs)
    if args.pages and args.page_slice:
        print(
            'Please specify either --pages or --page-slice, not both',
            file=sys.stderr,
        )
        sys.exit(1)
    if args.pages:
        args.pages = [p for p in args.pages if p <= num_whole_pdf_pages and p > 0]
        docs = [doc for doc in docs if doc.metadata['page_number'] in args.pages]
    if args.page_slice:
        args.pages = list(x + 1 for x in list(range(num_whole_pdf_pages))[convert_str_slice_notation_to_slice(args.page_slice)])
        docs = [doc for doc in docs if doc.metadata['page_number'] in args.pages]
    texts = [doc.page_content for doc in docs]
    all_text = '\n'.join(texts)
    word_count = get_word_count((all_text))
    print(
        f'Loaded {len(docs)} pages. Word count: {word_count} Char count: {len(all_text)}',
        file=sys.stderr,
    )
    if args.merge:
        from langchain.docstore.document import Document

        merged = Document(
            page_content=all_text,
            metadata={
                k: v for k, v in docs[0].metadata.items() if k not in {'page_number'}
            },
        )
    if args.split or word_count > args.chunk_size * 0.75:
        needs_splitting = True
    else:
        needs_splitting = False
    deliver_prompts(
        what=args.what,
        documents=[merged] if args.merge else docs,  # type: ignore
        needs_splitting=needs_splitting,
        copy=args.copy,
        edit=args.edit,
        chunk_size=args.chunk_size,
        extra_chunk_info_fn=pymupdf_doc_page_info,
        dry_run=args.dry_run,
        parts=args.parts,
    )


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

DEFAULT_PDF_WHAT = 'the content of a PDF file'
DEFAULT_URL_WHAT = 'the content of a webpage'
DEFAULT_HTML_WHAT = 'the text content of a html file'
DEFAULT_GENERAL_WHAT = 'the content of a document'



import argparse
from . import __version__


def get_get_prompt_base_arg_parser(description: str) -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )

    parser.add_argument(
        '-V',
        '--version',
        action='version',
        version=f'%(prog)s {__version__}',
    )
    parser.add_argument(
        '-c', '--copy', help='Copy the prompt to clipboard', action='store_true'
    )
    parser.add_argument(
        '-e', '--edit', help='Edit the prompt and copy manually', action='store_true'
    )
    parser.add_argument(
        '-m',
        '--model',
        help='Model to use',
        metavar='model',
        type=str,
        default='gpt-3.5-turbo',
    )
    parser.add_argument(
        '-S',
        '--split',
        help='Split the prompt into multiple parts',
        action='store_true',
    )
    parser.add_argument(
        '-s',
        '--chunk-size',
        help='Chunk size when splitting transcript, also used to determine whether to split',
        metavar='chunk_size',
        type=int,
        default=2000,
    )
    parser.add_argument(
        '-P',
        '--parts',
        help='Parts to select in the processes list of Documents',
        type=int,
        nargs='+',
        default=None,
    )
    parser.add_argument('-n', '--dry-run', help='Dry run', action='store_true')
    return parser
